;
; PGA driver for daisychained PGA2310/PGA2311/PGA4311 (maybe wolfson or other gain control chip)
;
; This include file provides following functions:
;    o PGA_Init:       initializes the connected PGA modules
;    o PGA_PinSet:     user function to set a pin to a given 12-bit value
;    o PGA_Pin7bitSet: user function to set a pin to a given 7-bit value
;    o PGA_GatePinSet: user function to set the gate pin
;
; The pins to which the first PGA module is connected have to be defined here:
;
#define PGA_LAT_CS	LATC	; The chip select pin CS#
#define PGA_TRIS_CS	TRISC	; is connected to Port C.5
#define PGA_PIN_CS	5	; (CANNOT be shared with other outputs!)
;
#define PGA_LAT_DIN	LATC	; The data input pin DIN
#define PGA_TRIS_DIN	TRISC	; is connected to Port C.4
#define PGA_PIN_DIN	4	; (can be shared with other outputs)
;
#define PGA_LAT_SCLK	LATD	; The shift clock input pin SCLK
#define PGA_TRIS_SCLK	TRISD	; is connected to Port D.3
#define PGA_PIN_SCLK	3	; (can be shared with other outputs)
;
; Number of stereo pairs connected (ie 1 for each 2311/2310, 2 for each 4311)
;
#define pairs 8
#define PGA_NUMBER_OF_CHANNEL pairs*2
#define start_volume d'37' ;This is close to 0db gain when using Log curve
#define start_balance d'64'
;
; ==========================================================================
;
; Copyright (C) 2005  Pilo Chambert (pilo.c@wanadoo.fr)
;  Modified a bit by Lyle Hazelwood (lylehaze@bellsouth.net) Feb 2007
;
; ==========================================================================
;
; This file is part of an MIOS application
;
; This application is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; This application is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this application; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
; ==========================================================================

PGA_Init
	;; enable pin drivers
	bcf	PGA_TRIS_CS, PGA_PIN_CS
	bcf	PGA_TRIS_DIN, PGA_PIN_DIN
	bcf	PGA_TRIS_SCLK, PGA_PIN_SCLK

	;; init gain to 0dB (192), MIDI CC#7,96
	movlw	16		; init outer loop counter
	movwf	PGA_SR_CTR

	lfsr	FSR0, MIDI_VOL_0
	movlw	start_volume
PGA_InitGain_Loop
	movwf	POSTINC0

	;; loop until last register reached
	decf	PGA_SR_CTR, F
	bnz	PGA_InitGain_Loop
	
	movlw	16		; init outer loop counter
	movwf	PGA_SR_CTR

	;init balance to centered, MIDI CC#8,64
	lfsr	FSR0, MIDI_BAL_0
	movlw	start_balance
PGA_InitBal_Loop
	movwf	POSTINC0

	;; loop until last register reached
	decf	PGA_SR_CTR, F
	bnz	PGA_InitBal_Loop

	;convert MIDI Volume and Balance settings to audio levels	
	call	Mid2Gain
	;and send those to the volume chips
	call	PGA_SEND_GAIN

;	return 		;from here if you don't want to send out the reset by MIDI
	
	;this will send our reset settings out MIDI to update the control deck
	call	MIOS_MIDI_BeginStream
	movlw	0xB0
	movwf	PGA_SR_CTR
loophere
	movf	PGA_SR_CTR,W
	call	MIOS_MIDI_TxBufferPut	;Control Change + Channel
	movlw	0x07					;MIDI VOLUME
	call	MIOS_MIDI_TxBufferPut
	movlw	start_volume
	call	MIOS_MIDI_TxBufferPut

	movf	PGA_SR_CTR,W
	call	MIOS_MIDI_TxBufferPut	;Control Change + Channel
	movlw	0x08					;MIDI BALANCE
	call	MIOS_MIDI_TxBufferPut
	movlw	start_balance			;Centered
	call	MIOS_MIDI_TxBufferPut

	incf	PGA_SR_CTR,F
	movlw	pairs + 0xB0
	cpfseq	PGA_SR_CTR
	bra		loophere
	call	MIOS_MIDI_EndStream
	
	return
;; --------------------------------------------------------------------------
;;  FUNCTION: PGA_SEND_GAIN
;;  DESCRIPTION: This function set for all channel (CHANNEL_GAIN_0 to CHANNEL_GAIN_31)
;;  IN:   o gain value in CHANNEL_GAIN_0 to CHANNEL_GAIN_31
;;  OUT:  -
;;  USES: BSR --- PGA_SR contains unknown values after the load procedure
;; --------------------------------------------------------------------------
PGA_SEND_GAIN
	IRQ_DISABLE				; disable interrupts
	;; (for the case that interrupt driven pins are used)

    bcf		PGA_LAT_SCLK, PGA_PIN_SCLK	; clear clock
	bcf		PGA_LAT_CS, PGA_PIN_CS	; activate chip select


	movlw	PGA_NUMBER_OF_CHANNEL		; init outer loop counter
	movwf	PGA_SR_CTR

	lfsr	FSR0, CHANNEL_GAIN_0
	movlw	PGA_NUMBER_OF_CHANNEL-1
	addwf	FSR0L, F

PGA_LoadSR_OuterLoop
	clrf	PGA_SR_BIT_CTR	; clear inner loop counter

	;; pointer = current pga channel (*PGA_SR = *CHANNEL_GAIN_3)
	movff	POSTDEC0, PGA_SR


PGA_LoadSR_Value
PGA_LoadSR_Value_Loop
	btfss	PGA_SR,7
	bcf	PGA_LAT_DIN, PGA_PIN_DIN	; set DIN depending on current MSB
	btfsc	PGA_SR, 7
	bsf	PGA_LAT_DIN, PGA_PIN_DIN

	rlncf	PGA_SR, F		; start to shift the 8-bit value

	bsf	PGA_LAT_SCLK, PGA_PIN_SCLK	; rising clock edge
	incf	PGA_SR_BIT_CTR, F	; loop 8 times
	bcf	PGA_LAT_SCLK, PGA_PIN_SCLK	; falling clock edge

	btfss	PGA_SR_BIT_CTR, 3 ; if PGA_SR_BIT_CTR reach 8 then go for next channel
	rgoto   PGA_LoadSR_Value_Loop

PGA_LoadSR_Next
	;; loop until last register reached
	decf	PGA_SR_CTR, F
	bnz	PGA_LoadSR_OuterLoop

	bsf	PGA_LAT_CS, PGA_PIN_CS	; deactivate chip select

	IRQ_ENABLE				; enable interrupts again
	return


;The SavePreset and LoadPreset were written before me.
;because we use MIDI_VOL and MIDI_BAL to generate the gain settings,
;saving the gain tables is no longer useful	
;PGA_SavePreset
;	TABLE_ADDR 0x7000	;  preset data stored at 0x7000 (flash memory)
;	lfsr	FSR1, CHANNEL_GAIN_0 ;  copy from addr CHANNEL_GAIN_0
;	call    MIOS_FLASH_Write ;  copy 64 bytes
;	return

;PGA_LoadPreset
;	TABLE_ADDR 0x7000                ; load address into TABLPTR[LH]
 ;       lfsr        FSR1, CHANNEL_GAIN_0 ; a free 64 byte buffer in RAM
;	call        MIOS_FLASH_Read
;	return 
